/**
 ******************************************************************************
 * @project  Arduino Web Server LED Control Example 
 * @author   Edward Ahn
 * @version  V1.1
 * @date     11-May-2016
 * @brief    Ethernet Communication
 * @MCU      ARM Cortex-M0+
*******************************************************************************
**/

#include <SPI.h>
#include <Ethernet2.h> // Ethernet 라이브러리를 보면 SPI를 사용하여 아두이노와 통신한다.

#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io는 MAC주소가 내장되어 있는 모듈입니다.
;
#else // WIZ550io모듈이 아니라면 MAC을 설정해주어야 한다.
byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED}; // mac 주소 입력
#endif

#define __USE_DHCP__ // DHCP사용 시 Define

IPAddress ip(192,168,1,177); // TCP/IP 네트워크에 있는 호스트를 식별하는 숫자
IPAddress gateway( 192, 168, 1, 1 ); // 다른 네트워크에 있는 호스트와 통신할 때 사용
IPAddress subnet( 255, 255, 255, 0 ); // 호스트가 로컬 or 원격 네트워크에 있는지 확인할 때 사용
IPAddress myDns(8, 8, 8, 8); // 구글에서 제공하는 DNS

// 네트워크 정보에 대한 자세한 내용은 아래 사이트 확인 
// https://support.microsoft.com/ko-kr/kb/164015

// 소켓을 80번 포트로 생성
// HTTP를 사용할 땐 기본적으로 80번 포트 사용.
// 아래 사이트는 IANA(Internet Assigned Numbers Authority)이며, IANA에서 HTTP는 포트를 80번을 사용하자고 권유하였음.
// 하지만 사용자가 원하는 대로 포트 설정 가능하다.
// https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.txt
EthernetServer server(80);

#define LED_PORT 13    // LED PORT 번호 정의

int LED_FLAG = 0;         // LED가 H/W적으로 ON 되었는지 확인하는 플래그

void setup() {
// 이더넷 디바이스 초기화
#if defined __USE_DHCP__ // DHCP 사용 시 (네트워크 정보 DHCP에서 할당받음)
#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io에 할당된 MAC 주소를 사용할 경우
  Ethernet.begin();
#else
  Ethernet.begin(mac); // MAC이 없는 이더넷 디바이스의 경우
#endif  
#else // Static 주소로 사용 시 (네트워크 정보 사용자가 임의로 지정)
#if defined(WIZ550io_WITH_MACADDRESS) 
  Ethernet.begin(ip, myDns, gateway, subnet);
#else
  Ethernet.begin(mac, ip, myDns, gateway, subnet);
#endif  
#endif 

  // 서버 연결 시작 (Socket open -> Listen 상태 돌입)
  
  server.begin();
  SerialUSB.begin(9600); // 시리얼 통신 초기 선언
  // WizArduino M0(Zero)는 컴파일할 때 BOOT모드로 변경되었다가 App으로 동작하게 된다. 
  // 이로인해 시리얼을 사용할 때 Delay 6초를 해준다. (업로드 시 MCU가 BOOT로 갔다가 App으로 넘어오는 시간이 6초이기 때문) 
  delay(6000); 
   while (!SerialUSB) {
    ; // 시리얼 포트가 연결 될 때까지 기다림, 레오나르도 보드의 경우만에만 필요
  }
  pinMode(LED_PORT, OUTPUT);    // LED PORT 출력 설정
  digitalWrite(LED_PORT, LOW); // LED PORT Defualt 'OFF'으로 설정 
 
  SerialUSB.println("WebServer Control LED");
  SerialUSB.print("server is at ");
  SerialUSB.println(Ethernet.localIP()); // 설정된 IP 시리얼 메세지로 출력
}

void loop() {
  EthernetClient client = server.available(); // ESTABLISHED & CLOSE_WAIT 상태인지 확인.
  // server.available()을 반복문으로 돌면서 클라이언트 연결이 올 때까지 대기.
  // 만약 클라이언트로 부터 연결(connect)이 오면 칩 내부적으로 자동 연결(accept)을 하고 ESTBLISHED상태로 넘어간다.
  // ESTABLISHED 상태가 되면 Connection이 되었다는 것을 의미.
  if (client) {
    SerialUSB.println("new client");
    // HTTP 데이터 처리를 위해 Flag 사용
    boolean currentLineIsBlank = true;
    String buffer = ""; // 문자열 버퍼 선언
    // 예외처리를 위한 서버의 현재 소켓 상태를 읽는다.
    // LISTEN, CLOSE, FIN_WAIT, CLOSE_WAIT 상태라면 Connection이라 판단하지 않는다.
    while (client.connected()) {
      if (client.available()) { // 클라이언트로 부터 수신된 데이터가 있다면
        char c = client.read(); // 데이터를 1byte씩 읽어서 c 변수에 저장
        buffer += c;  // 1 byte 데이터를 문자열 버퍼에 저장
        SerialUSB.write(c); // 클라이언트로 부터 수신한 데이터 시리얼로 출력
        // 클라이언트로 부터 HTTP 요청 데이터 중 2번의 빈 라인(\n)을 받으면 HTTP 데이터 종료, 표준 HTTP 응답 헤더 전송 시작
        if (c == '\n' && currentLineIsBlank) { // currentLineIsBlank이 True일 때
          // HTTP Message
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println();
          
          // 서버에 지정되어 있는 데이터 클라이언트로 전송
          // HTML Message
          client.println("<!DOCTYPE HTML>"); // HTML5 사용시
          client.println("<html>");  // HTML 언어로 코드 작성
          client.println("<body>");
          // LED 상태 확인
          if(LED_FLAG == 1){    // LED PORT가 '1'이면 LED ON, 아니면 LED OFF
            client.println("LED is <font color='blue'>ON</font>"); // LED ON
          }
          else {
            client.println("LED is <font color='red'>OFF</font>"); // LED OFF
          }
          // 웹 페이지에서 LED ON/OFF 제어 할 수 있게 나타낸다.
          client.println("<br />");
          client.println("<FORM method=\"get\" action=\"/led.cgi\">");
          client.println("<P> <INPUT type=\"radio\" name=\"status\" value=\"1\">ON");
          client.println("<P> <INPUT type=\"radio\" name=\"status\" value=\"0\">OFF");
          client.println("<P> <INPUT type=\"submit\" value=\"Submit\"> </FORM>");
          client.println("</body>");
          client.println("</html>");
          // HTML Message Done
          break;
        }
        // '\n' 메세지가 오면 플래그 True
        // 플래그 True 되었으면 다음 요청 메세지 받을 수 있음.
        // 요청 메세지는 한 줄씩 받아서 저장하고 HTTP 데이터 마지막에 '\r\n'가 2번 오면 HTTP요청 메세지가 종료된다.
        if (c == '\n') {       
          currentLineIsBlank = true;
          //buffer="";
        }
        // '\r' 메세지가 온 이후부터 "GET /led.cgi?status=1" 문자열을 검색한다.
        // LED ON
        else if ( c == '\r') {
          if(buffer.indexOf("GET /led.cgi?status=1")>=0){
            digitalWrite(LED_PORT, HIGH); // LED ON
            LED_FLAG = 1;
            // HTTP
            client.println("HTTP/1.1 200 OK");
            client.println("Content-Type: text/html");
            client.println();
            // HTML
            client.println("<html>");
            client.println("<body>");
 
            if(LED_FLAG == 1){ // LED_FLAG값을 읽어 ON인지 OFF인지 웹페이지 상으로 출력한다.
              client.println("LED is <font color='blue'>ON</font>");
            } 
            else {
              client.println("LED is <font color='red'>OFF</font>");
            }

            client.println("<br />");
            client.println("<a href=\"/led.htm\">Go to control-page</a>"); // Main 페이지로 리턴 
            client.println("</body>");
            client.println("</html>");
            // HTML DONE
            currentLineIsBlank = false;
            break;
          }

        // '\r' 메세지가 온 이후부터 "GET /led.cgi?status=0" 문자열을 검색한다.
        // LED OFF
          if(buffer.indexOf("GET /led.cgi?status=0")>=0){
            digitalWrite(LED_PORT, LOW); // LED OFF
            LED_FLAG = 0;
            // HTTP
            client.println("HTTP/1.1 200 OK");
            client.println("Content-Type: text/html");
            client.println();
            // HTML
            client.println("<html>");
            client.println("<body>");
              // LED 상태 확인
            if(LED_FLAG == 1){    // LED PORT가 '1'이면 LED ON, 아니면 LED OFF
              client.println("LED is <font color='blue'>ON</font>");
            } 
            else {
              client.println("LED is <font color='red'>OFF</font>");
            }

            client.println("<br />");
            client.println("<a href=\"/led.htm\">Go to control-page</a>"); // Main 페이지로 리턴 
            client.println("</body>");
            client.println("</html>");
            // HTML DONE
            currentLineIsBlank = false;
            break;
          }
        }
        // '\r' 메세지가 아니면 플래그 False.
        // 플래그가 False면 응답메세지 보내지 않음.
        else{ //if( c != '\r') { 
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }
    }
    
    delay(1);
    client.stop(); // TCP Connection 종료
    SerialUSB.println("client disonnected");
  }
}
